require "active_support/all"
require "bunny"
require "yaml"

require "banter/configuration"
require "banter/context"
# require "banter/db_logger"
require "banter/rabbit_logger"
require "banter/message"
require "banter/publisher"
require "banter/server/rabbit_mq_subscriber"
require "banter/version"
require "banter/exceptions/payload_validation_error"
require 'banter/server'
require 'banter/subscriber'
require 'banter/notifier'
require 'banter/delay_publish_model'

if defined?(Rails::Railtie)
  require "banter/middleware"
  require "banter/railtie"
end

module Banter

# This method publishes payload to rabbitmq.  All listeners with appropriate
# routing keys will receive the payload.
# @param [String] routing_key Identifier of the message type
# @param [Hash] payload The data that will be passed to the subscriber

  def self.publish(routing_key, payload)
    Publisher.instance.publish(Banter::Context.instance, routing_key, payload)
  end

  # Delay actual publishing of messages to rabbitmq. Any messages published using Banter.publish will be delayed till
  # after the block. if any errors happen in this block, then no messages are published to the rabbitmq.
  # Usage:
  # Banter.delay_messages do
  #   Banter.publish('foo.bar', {}) # -> message will not be published here
  # end # -> message will be published here
  def self.delay_messages
    Publisher.instance.delay_messages{ yield }
  end

  # @return [Logger] Logger used for logging through the Banter gem
  def self.logger
    return Banter::Configuration.logger if Banter::Configuration.logger
    @logger ||= ActiveSupport::TaggedLogging.new(Logger.new($stdout))
  end

  # @param [Logger] logger Logger for publish, message received and error events
  def self.logger=(logger)
    Banter::Configuration.logger = ActiveSupport::TaggedLogging.new(logger)
  end

  # A convenience methods, specially for non-rails applications to configure Banter
  # Usage:
  # Banter.configure do |config|
  #    config.default_queue_ttl = 2.hours
  # end

  def self.configure
    yield Banter::Configuration
  end
end