module Banter
  module Configuration
    @@conf = nil

    # Configuration setting to specify the number of seconds a message will live in the queue before
    # being either consumed or discarded.
    # @default 1 hour
    mattr_accessor :default_queue_ttl
    @@default_queue_ttl = 36000 # 10 hours in seconds

    # The exchange to publish (and subscribe) on RabbitMQ
    # @default true
    mattr_accessor :exchange_name
    @@exchange_name = "banter"

    mattr_accessor :dead_letter_exchange_name
    @@dead_letter_exchange_name = "banter-dlx"

    mattr_accessor :dead_letter_queue_name
    @@dead_letter_queue_name = "deadletter"

    # The topic prefix for RabbitMQ
    # @default banter
    mattr_accessor :topic_prefix
    @@topic_prefix = "banter"

    mattr_accessor :durable_queues
    @@durable_queues = true

    mattr_accessor :logger

    mattr_accessor :notifier

    # RabbitMQ push and subscribe logging toggle
    # @default true
    mattr_accessor :logging_enabled
    @@logging_enabled = true

    # RabbitMQ push enabled
    # @default true
    mattr_accessor :push_enabled
    @@push_enabled = true

    # RabbitMQ batches messages and delays until the end of a request
    # @default false
    mattr_accessor :batch_messages
    @@batch_messages = false


    mattr_accessor :application_name
    @@application_name = nil

    # default pool size for the subscribers
    mattr_accessor :pool_size
    @@pool_size = 100

    def self.configure_with(environment_name, yaml_file = nil)
      @@yaml_file = yaml_file.nil? ? "config/banter.yml" : yaml_file
      yaml_data = YAML.load(ERB.new(File.read(@@yaml_file)).result)
      @@all_conf = Hashie::Mash.new(yaml_data)
      @@conf = @@all_conf[environment_name]
      self
    end

    def self.configuration
      self.configure_with(self.environment) if @@conf.nil?
      @@conf
    end

    def self.connection
      configuration[:connection]
    end

    def self.environment
      val = ENV["RAILS_ENV"] || ENV["RACK_ENV"]
      val = if val.present?
        val.to_sym
      else
        if defined?(Rails)
          Rails.env
        else
          raise "No environment can be found for configuration!"
        end
      end
    end

  end
end
