
require "hashie"
require "json"

module Banter
  class Message
    @@message_version = "1"
    attr_reader :ts
    attr_reader :pub
    attr_reader :version
    attr_reader :payload
    attr_reader :context
    attr_reader :dead_letter
    attr_reader :envelope

    def initialize
    end

    # Context object should be passed into the call.
    def serialize(context, routing_key, payload)
      @ts = Time.now.to_f
      @pub = "#{routing_key}:#{Socket.gethostname()}:#{Process::pid}"
      @version = @@message_version
      @payload = payload
      @context = context
      @dead_letter = nil
      to_hash
    end

    def parse(envelope)
      contents = Hashie::Mash.new(::JSON.parse(envelope))
      from_hash(contents)
      to_hash
    rescue => e
      ::Hashie::Mash.new({
        ts: Time.now.to_f,
        pub: "unknown",
        v: "unknown",
        payload: envelope,
        context: { "format" => "invalid" }.as_json
      })

    end

    def from_hash(contents)
      @ts = contents.ts
      @pub = contents.pub
      # Version used for messaging can get updated if we need to add extra header information, and need
      # to move each section of the library separately.
      @version = contents.v
      @payload = contents.payload
      @dead_letter = contents.dead_letter
      @context = ::Banter::Context.from_json(contents[:context])
      self
    end

    def to_hash
      ::Hashie::Mash.new({
        ts: @ts,
        pub: @pub,
        v: @@message_version,
        payload: @payload,
        dead_letter: @dead_letter,
        context: @context.as_json
      })

    end

    def retry_completed?
      @dead_letter.present? && @dead_letter.completed.present? && @dead_letter.completed > 0
    end

  end
end
