# encoding: utf-8
# Internal log class that is used to log envelopes before sending, after receiving, and failure to send
module Banter

  class RabbitLogger
    @@log_map = [:debug, :info, :warn, :error, :fatal]
    def self.enabled?
      true
      # Banter::Configuration.logging_enabled
    end

    def self.log_publish(routing_key, envelope)
      internal_log(:warn, "BANTER PUBLISH", envelope, routing_key, envelope[:payload])
    end

    def self.failed_publish(routing_key, properties, envelope)
      internal_log(:error, "BANTER FAILED_SEND", envelope, routing_key, { properties: properties, payload:envelope[:payload] })
    end

    def self.log_receive(routing_key, envelope)
      internal_log(:warn, "BANTER RECEIVED", envelope, routing_key, envelope[:payload])
    end

    def self.log_complete(routing_key, envelope)
      internal_log(:warn, "BANTER COMPLETED", envelope, routing_key, envelope[:payload])
    end

    def self.log_subscriber_failed(routing_key, delivery_info, properties, envelope, contents)
      if envelope.nil?
        # message could not be parsed, so we have to log this one differently.
        tags = ["BANTER SUBSCRIBER FAILED", Time.now.to_i.to_s, routing_key].map{|x| utf(x)}
        logger.tagged(tags) { logger.send(:error, utf( "contents: #{contents.to_s} -- properties: #{properties.to_s}")) }
      else
        internal_log(:warn, "BANTER SUBSCRIBER FAILED", envelope, routing_key,  { delivery_info: delivery_info, properties: properties,
          contents: envelope[:payload] } )
      end
    end

    def self.log_service(service_name, message)
      return unless enabled?
      tags = ["BANTER SERVICE", service_name, Process::pid]
      logger.tagged(tags) { logger.info message.as_json}
    end

    def self.log(log_level, message)
      tags = ["BANTER LOG_LEVEL:#{@@log_map[log_level].capitalize.to_s}", Process::pid.to_s].map{|x| utf(x)}
      logger.tagged(tags) { logger.send(@@log_map[log_level], utf(message.as_json.to_s) ) }
    end

    def self.internal_log(log_level, log_type, envelope, routing_key, payload)
      return unless enabled?
      tags = [log_type, envelope[:ts].to_s, envelope[:pub], envelope[:v].to_s, routing_key].compact.map{|x| utf(x)}
      logger.tagged(tags) { logger.send(log_level, utf(payload.to_s)) }
    end

    def self.logger
      Banter.logger
    end

    def self.utf(field)
      field.force_encoding('utf-8')
    end

  end
end
