# This railtie provides Rails engine hooks
require 'banter/middleware'
require 'banter/configuration'

module Banter
  class Railtie < ::Rails::Railtie
    config.banter = Banter::Configuration

    initializer "banter.insert_middleware" do |app|
      app.config.middleware.use Banter::Middleware
    end

    config.before_configuration do
      Banter::Configuration.application_name = Rails.application.class.name.to_s.gsub("::Application", '').downcase
    end

    config.after_initialize do
      Banter.logger = config.banter.logger if config.banter.logger

    end

    console do
      # Set up basic context to identify messages getting generated from console
      Banter::Context.setup_context(
        application: "#{Banter::Configuration.application_name}/console",
        hostname: `hostname`.strip,
        unique_id: Digest::SHA1.new.to_s
      )
    end

    rake_tasks do
      # Set up basic context to identify messages getting generated from rake tasks
      Banter::Context.setup_context(
        application: "#{Banter::Configuration.application_name}/rake",
        hostname: `hostname`.strip,
        unique_id: Digest::SHA1.new.to_s
      )
    end


  end
end
