require 'rubygems'
require 'celluloid/autostart'
require 'celluloid/io'
require 'active_support/all'
require 'optparse'
require 'fileutils'
require 'singleton'

require_relative './client_queue_listener'

# In order to use the server, the caller must be able to bring in the necessary
# classes for require before instantiating the instance.

module Banter
  module Server
    class SubscriberServer
      include Celluloid

      attr_reader :workers, :process_name, :pid, :hostname, :banter_worker_id

      def self.instance
        @instance ||= new
      end

      def set_workers(subscribers)
        @workers = subscribers.map { |subscriber| create_queue_listeners(subscriber) }
      end

      def start
        set_information
        start_workers
        handle_interrupts
        handle_teardown
      ensure
        stop_worker
      end

      private

      def handle_teardown
        ::Banter::RabbitLogger.log_service("all_services", "Starting shutdown of all services")

        workers.each do |worker|
          ::Banter::RabbitLogger.log_service("all_services", "Tearing down worker: #{worker.worker_class.name}")
          begin
            STDOUT.puts "Tearing down subscriber for #{worker.worker_class.name}"
            worker.shutdown
          rescue => e
            ::Banter::RabbitLogger.log_service("all_services", "#{worker.worker_class.name} - did not tear down correctly.  Error - #{e.message}")
          end
        end
      end

      def handle_interrupts
        thread     = Thread.current
        interrupts = ["HUP", "INT", "QUIT", "ABRT", "TERM"]
        interrupts.each do |signal_name|
          Signal.trap(signal_name) {
            puts "Processing #{signal_name}"
            thread.run
          }
        end

        Thread.stop
      end

      def start_workers
        workers.each do |worker|
          puts "starting worker #{worker.inspect}"
          worker.start
        end
      end

      def stop_worker
        Banter::CLI.instance.remove_pid
      end

      def set_information
        @process_name = $0
        @pid          = Process.pid
        @hostname     = `hostname -f`.chomp
      end

      def create_queue_listeners(subscriber_class)
        ClientQueueListener.new(subscriber_class)
      end
    end
  end
end

